--[[
	vBagnon\category.lua
		Category frames for Bagnon windows
		
		A category frame is a frame that contains a set if item buttons
		defined by a specific rule
		
		Category frames have the following:
			a rule (nil for all items)
			a list of items
			a title

		BagnonCat.Get() should be used whenever a category frame is needed
--]]

--[[ Locals ]]--

--local msg = function(msg) ChatFrame1:AddMessage(msg or 'nil', 0.5, 1, 1) end

local function GetSize(bag)
	if Bagnon_IsBankBag(bag) and not Bagnon_PlayerAtBank() then
		return 0
	elseif bag == KEYRING_CONTAINER then
		return GetKeyRingSize()
	end
	return GetContainerNumSlots(bag)
end

local function SlotToIndex(bag, slot)
	if bag > 0 then
		return bag * 100 + slot
	else
		return bag * 100 - slot
	end
end

local function IndexToSlot(index)
	local slot = mod(abs(index), 100)
	if index > 0 then
		return (index - slot)/100, slot
	else
		return (index + slot)/100, slot
	end
end

--[[ A BagnonCat Object ]]--

local Frame = CreateFrame("Frame")
local Frame_mt = {__index = Frame}

--[[ Update ]]--

--set's a frame to the given parameters
function Frame:Set(parent, rule)
--	msg('set ' .. self.id)

	if rule and type(rule) == 'string' then
		self.rule = Bagnon_StringToRule(rule)
	else
		self.rule = rule
	end

	self:SetParent(parent)
	self:SetAlpha(parent:GetAlpha())
	self:SetFrameLevel(1)
	
	self:UpdateAllSlots()
end

--sticks the frame into the unused list, frees all items on the frame
function Frame:Release()
--	msg('release ' .. self.id)
	
	self.rule = nil
	self:RemoveAllItems()
	TPool.Release(self, 'BagnonCat')
end

--[[ Item Functions ]]--

--adds a specific item to the frame
function Frame:AddItem(bag, slot)
	local index = SlotToIndex(bag, slot)
	local items = self.items
	local item = items[index]

	if not item then
		item = BagnonItem.Get()
		item:Set(self, bag, slot)

		items[index] = item
		self.count = (self.count or 0) + 1
	end
	item:Update()
end

--releases a specific item from the frame
function Frame:RemoveItem(bag, slot, update)
	local index = SlotToIndex(bag, slot)
	local item = self.items[index]

	if item then
		self.items[index] = nil
		item:Release()
		
		if self.count then
			self.count = self.count - 1
		end
		
		if update then
			self:GetParent().sizeChanged = true
		end
	end
end

--releases all items from the frame, and clears the frame's list of items
function Frame:RemoveAllItems()
--	msg('remove all items ' .. self.id)
	
	if next(self.items) then
		for i, item in pairs(self.items) do
			self.items[i] = nil
			item:Release()
		end
		self:GetParent().sizeChanged = true	
	end
	
	self:SetWidth(0)
	self:SetHeight(0)
	self.count = nil
end


--[[ Bag Functions ]]--

function Frame:AddBag(bag)
	local oldCount = self.count
	local rule = self.rule
	
	if rule then
		for slot = 1, GetSize(bag) do
			if rule(GetContainerItemLink(bag, slot), bag) then
				self:AddItem(bag, slot)
			end
		end
	end

	if self.count ~= oldCount then
		self:GetParent().sizeChanged = true
	end
end

function Frame:RemoveBag(bag)
	for i in pairs(self.items) do
		local iBag, iSlot = IndexToSlot(i)
		if iBag == bag then
			self:RemoveItem(iBag, iSlot)
		end
	end

	if self.count ~= oldCount then
		self:GetParent().sizeChanged = true
	end
end

--[[ Update Functions ]]--

function Frame:UpdateSlot(bag, slot, itemLink)
	local oldCount = self.count
	local rule = self.rule

	if self:GetParent():HasBag(bag) and rule and rule(itemLink, bag) then
		self:AddItem(bag, slot)
	else
		self:RemoveItem(bag, slot)
	end

	if self.count ~= oldCount then
		self:GetParent().sizeChanged = true
	end
end

--cycles through all possible slots
function Frame:UpdateAllSlots()
--	msg('update all slots ' .. self.id)
	
	local rule = self.rule
	local oldCount = self.count

	local bags = self:GetParent():GetBags()
	for _, bag in ipairs(bags) do
		for slot = 1, GetSize(bag) do
			if rule and rule(GetContainerItemLink(bag, slot), bag) then
				self:AddItem(bag, slot)
			else
				self:RemoveItem(bag, slot)
			end
		end
	end

	if self.count ~= oldCount then
		self:GetParent().sizeChanged = true
	end
end

function Frame:UpdateLock(bag, slot, locked)
	local item = self.items[SlotToIndex(bag, slot)]
	if item then
		item:UpdateLock(locked)
	end
end

function Frame:UpdateBorders()
	for _, item in pairs(self.items) do
		item:UpdateBorder()
	end
end

--[[ Highlighting ]]--

function Frame:HighlightRule(rule, highlight)
	if highlight and rule then
		for _, item in pairs(self.items) do
			local bag = item:GetParent():GetID()
			local slot = item:GetID()

			if rule(GetContainerItemLink(bag, slot), bag) then
				item:Unfade(true)
			else
				item:Fade()
			end
		end
	else
		for _, item in pairs(self.items) do
			item:Unfade()
		end
	end
end

function Frame:HighlightBag(bag, highlight)
	if highlight then
		for _, item in pairs(self.items) do
			if item:GetParent():GetID() == bag then
				item:Unfade(true)
			else
				item:Fade()
			end
		end
	else
		for _, item in pairs(self.items) do
			item:Unfade()
		end
	end
end

--[[ Config ]]--

function Frame:Layout()
--	msg('layout ' .. self.id)
	
	local columns, spacing = self:GetParent():GetLayout()
	local itemSize = (BagnonItem.SIZE + spacing*2)
	local offset = spacing / 2
	local i = 0
	local items = self.items

	local bags = self:GetParent().bags
	for _, bag in ipairs(bags) do
		local bagSize = GetSize(bag)

		for slot = 1, bagSize do
			local item = items[SlotToIndex(bag, slot)]
			if item then
				i = i + 1
				local row = mod(i - 1, columns)
				local col = ceil(i / columns) - 1
				item:SetPoint("TOPLEFT", self, "TOPLEFT", itemSize * row + offset, -itemSize * col - offset - (self.paddingY or 0))
			end
		end
	end

	if i > 0 then
		self:SetWidth(itemSize * math.min(columns, i) - spacing)
		self:SetHeight(itemSize * ceil(i / columns) - spacing + (self.paddingY or 0))
	else
		self:SetWidth(0); self:SetHeight(0)
	end
end

--[[ Object Constructor ]]--

local function CreateCategoryFrame(lastCreated)
	local name = "BagnonCat" .. lastCreated
	local frame = CreateFrame("Frame", name, UIParent)
	setmetatable(frame, Frame_mt)

	frame.paddingY = 6
	frame.items = {}

	return frame
end

function BagnonCat_Get()
	return TPool.Get('BagnonCat', CreateCategoryFrame)
end