--[[
	vBagnon\item.lua
		An item button
--]]

local UPDATE_DELAY = 0.3
local lastUsed = 0
local SLOT_INFO = 'bag %d, slot %d'


local function AddExtraInfo(bag, slot)
	GameTooltip:AddLine("bag: " .. bag .. ", slot: " .. slot)
	if GetContainerItemLink(bag, slot) then
		local _, _, id = string.find(GetContainerItemLink(bag, slot), "item:(%d+):")
		local itemName, _, itemQuality, _, itemLevel, itemType, itemSubType, _, itemEquipLoc = GetItemInfo(id)
		GameTooltip:AddLine("id: " .. id)
		GameTooltip:AddLine("quality: " .. (itemQuality or "nil"))
		GameTooltip:AddLine("level: " .. (itemLevel or "nil"))
		GameTooltip:AddLine("type: " .. (itemType or "nil"))
		GameTooltip:AddLine("subType: " .. (itemSubType or "nil"))
		GameTooltip:AddLine("loc: " .. (itemEquipLoc or "nil"))
	end
	GameTooltip:Show()
end

--[[ Item Object ]]--

local Item = CreateFrame('Button')
local Item_mt = {__index = Item}

local function DummyBag_Get(frame, bag)
	local bagFrame = getglobal(frame:GetName() .. bag)
	if not bagFrame then
		bagFrame = CreateFrame('Frame', frame:GetName() .. bag, frame)
		bagFrame:SetID(bag)
		bagFrame:SetAlpha(frame:GetAlpha())
	end
	return bagFrame
end

function Item:Set(parent, bag, slot)
	self:SetParent(DummyBag_Get(parent, bag))
	self:SetAlpha(parent:GetAlpha())
	self:SetFrameLevel(1)
	self:SetID(slot)
end

function Item:Release()
	self:SetID(-99)
	self.hasItem = nil
	self.isLink = nil
	TPool.Release(self, 'BagnonItem')
end

--[[ Update Functions ]]--

-- Update the texture, lock status, and other information about an item
function Item:Update()
	local texture, itemCount, locked, _, readable = GetContainerItemInfo(self:GetParent():GetID(), self:GetID())

	self.readable = readable
	if texture then
		self.hasItem = 1
	else
		self.hasItem = nil
	end

	SetItemButtonDesaturated(self, locked)
	SetItemButtonTexture(self, texture)
	SetItemButtonCount(self, itemCount)
	
	self:UpdateBorder()
	self:UpdateCooldown()
end

function Item:UpdateBorder()
	local itemName = self:GetName()
	local bag = self:GetParent():GetID()
	local slot = self:GetID()
	
	if BagnonSets.qualityBorders then
		if self.hasItem then
			local hexString = string.match(GetContainerItemLink(bag , slot) ,"|cff([%l%d]+)|H")
			local r = tonumber(strsub(hexString, 1, 2), 16)/256
			local g = tonumber(strsub(hexString, 3, 4), 16)/256
			local b = tonumber(strsub(hexString, 5, 6), 16)/256
			if not(r == g and r == b) then
				getglobal(itemName .. "Border"):SetVertexColor(r, g, b, 0.5)
				getglobal(itemName .. "Border"):Show()
			else
				getglobal(itemName .. "Border"):Hide()
			end
		else
			getglobal(itemName .. "Border"):Hide()
		end
	else
		getglobal(itemName .. "Border"):Hide()
	end

	if bag == KEYRING_CONTAINER then
		getglobal(itemName .. "NormalTexture"):SetVertexColor(1, 0.7, 0)
	elseif Bagnon_IsAmmoBag(bag) then
		getglobal(itemName .. "NormalTexture"):SetVertexColor(1, 1, 0)
	elseif Bagnon_IsProfessionBag(bag) then
		getglobal(itemName .. "NormalTexture"):SetVertexColor(0, 1, 0)
	else
		getglobal(itemName .. "NormalTexture"):SetVertexColor(1, 1, 1)
	end
end

function Item:UpdateLock(locked)
	SetItemButtonDesaturated(self, locked)
end

function Item:UpdateCooldown()
	local cooldown = getglobal(self:GetName().."Cooldown")
	if (not self.isLink) and self.hasItem then
		local start, duration, enable = GetContainerItemCooldown(self:GetParent():GetID(), self:GetID())
		CooldownFrame_SetTimer(cooldown, start, duration, enable)
	else
		CooldownFrame_SetTimer(cooldown, 0, 0, 0)
	end
end

--[[ OnX Functions ]]--

function Item:OnEnter()
	local bag = self:GetParent():GetID()
	local slot = self:GetID()

	if self.isLink then
		if self.hasItem then
			Bagnon_AnchorTooltip(self)
			
			local player = self:GetParent():GetParent().player
			local link, count = BagnonDB.GetItemData(player, bag, slot)
			
			GameTooltip:SetHyperlink(link, count)
		end
	else
		--blizzard totally needs to make the bank item not a special case
		if self:GetParent():GetID() == -1 then
			Bagnon_AnchorTooltip(self)
			GameTooltip:SetInventoryItem('player', BankButtonIDToInvSlotID(self:GetID()))
		else
			ContainerFrameItemButton_OnEnter(self)
		end
--		AddExtraInfo(bag, slot)
	end

	if not self.hasItem then
		GameTooltip:AddLine(format(SLOT_INFO, bag, slot))
		GameTooltip:Show()
	end
end

function Item:OnLeave()
	self.updateTooltip = nil
	GameTooltip:Hide()
	ResetCursor()
end

function Item:OnHide()
	self:Unfade()
	if self.hasStackSplit and self.hasStackSplit == 1 then
		StackSplitFrame:Hide()
	end
end

function Item:Fade()
	local parent = self:GetParent()
	if parent then
		self:SetAlpha(parent:GetAlpha() / 3)
	end
	self:UnlockHighlight()
end

function Item:Unfade(highlight)
	local parent = self:GetParent()
	if parent then
		self:SetAlpha(parent:GetAlpha())
	else
		self:SetAlpha(1)
	end

	if highlight and not GetContainerItemInfo(self:GetParent():GetID(), self:GetID()) then
		self:LockHighlight()		
	else
		self:UnlockHighlight()
	end
end

--[[ Constructor ]]--

local function OnUpdate()
	if not this.isLink and GameTooltip:IsOwned(this) then
		if not this.elapsed or this.elapsed < 0 then
			local enable = select(3, GetContainerItemCooldown(this:GetParent():GetID(), this:GetID()))
			if enable == 1 then
				this:OnEnter()
			end
			this.elapsed = UPDATE_DELAY
		else
			this.elapsed = this.elapsed - arg1
		end
	end
end

local function OnEnter() 
	this.elapsed = nil
	this:OnEnter() 
end

local function OnLeave() 
	this.elapsed = nil 
	this:OnLeave() 
end

local function OnHide()
	this:OnHide() 
end

local function Item_Create(lastCreated)
	local name = "BagnonItem" .. lastCreated
	local item = CreateFrame('Button', name, nil, 'ContainerFrameItemButtonTemplate')
	setmetatable(item, Item_mt)
	item:ClearAllPoints()

	local borderTexture = item:CreateTexture(name .. "Border", "OVERLAY")
	borderTexture:SetWidth(67)
	borderTexture:SetHeight(67)
	borderTexture:SetPoint("CENTER", item)
	borderTexture:SetTexture("Interface\\Buttons\\UI-ActionButton-Border")
	borderTexture:SetBlendMode("ADD")
	borderTexture:Hide()
	
	item:UnregisterAllEvents()
	item:SetScript('OnEvent', nil)

	item:SetScript("OnEnter", OnEnter)
	item:SetScript("OnLeave", OnLeave)
	item:SetScript("OnUpdate", OnUpdate)
	
	item:SetScript("OnHide", OnHide)

	return item
end

BagnonItem = {
	SIZE = 37,
	
	Get = function() return TPool.Get('BagnonItem', Item_Create) end
}